@@ -160,6 +160,8 @@ typedef struct
 
 	GtkTextTag *invalid_char_tag;
 
+	gint64 insertion_count;
+
 	guint has_draw_spaces_tag : 1;
 	guint highlight_syntax : 1;
 	guint highlight_brackets : 1;
@@ -1054,6 +1056,8 @@ gtk_source_buffer_real_insert_text (GtkTextBuffer *buffer,
 				    const gchar   *text,
 				    gint           len)
 {
+	GtkSourceBuffer *source_buffer = (GtkSourceBuffer *)buffer;
+	GtkSourceBufferPrivate *priv = gtk_source_buffer_get_instance_private (source_buffer);
 	gint start_offset;
 
 	g_return_if_fail (GTK_SOURCE_IS_BUFFER (buffer));
@@ -1070,6 +1074,8 @@ gtk_source_buffer_real_insert_text (GtkTextBuffer *buffer,
 	 */
 	GTK_TEXT_BUFFER_CLASS (gtk_source_buffer_parent_class)->insert_text (buffer, iter, text, len);
 
+	priv->insertion_count++;
+
 	gtk_source_buffer_content_inserted (buffer,
 					    start_offset,
 					    gtk_text_iter_get_offset (iter));
@@ -1563,11 +1569,10 @@ _gtk_source_buffer_update_search_highlight (GtkSourceBuffer   *buffer,
 					    gboolean           synchronous)
 {
 	GtkSourceBufferPrivate *priv = gtk_source_buffer_get_instance_private (buffer);
-	GList *l;
 
 	g_return_if_fail (GTK_SOURCE_IS_BUFFER (buffer));
 
-	for (l = priv->search_contexts; l != NULL; l = l->next)
+	for (GList *l = priv->search_contexts; l != NULL; l = l->next)
 	{
 		GtkSourceSearchContext *search_context = l->data;
 
@@ -1578,6 +1583,24 @@ _gtk_source_buffer_update_search_highlight (GtkSourceBuffer   *buffer,
 	}
 }
 
+gboolean
+_gtk_source_buffer_has_search_highlights (GtkSourceBuffer *buffer)
+{
+	GtkSourceBufferPrivate *priv = gtk_source_buffer_get_instance_private (buffer);
+
+	g_return_val_if_fail (GTK_SOURCE_IS_BUFFER (buffer), FALSE);
+
+	for (GList *l = priv->search_contexts; l != NULL; l = l->next)
+	{
+		GtkSourceSearchContext *search_context = l->data;
+
+		if (gtk_source_search_context_get_highlight (search_context))
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
 /**
  * gtk_source_buffer_ensure_highlight:
  * @buffer: a #GtkSourceBuffer.
@@ -3034,3 +3057,13 @@ _gtk_source_buffer_has_spaces_tag (GtkSourceBuffer *buffer)
 
 	return priv->has_draw_spaces_tag;
 }
+
+gint64
+_gtk_source_buffer_get_insertion_count (GtkSourceBuffer *buffer)
+{
+	GtkSourceBufferPrivate *priv = gtk_source_buffer_get_instance_private (buffer);
+
+	g_return_val_if_fail (GTK_SOURCE_IS_BUFFER (buffer), FALSE);
+
+	return priv->insertion_count;
+}
