@@ -1,8 +1,7 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
- * gtksourcegutterrendererpixbuf.c
+/*
  * This file is part of GtkSourceView
  *
- * Copyright (C) 2010 - Jesse van den Kieboom
+ * Copyright 2010 - Jesse van den Kieboom
  *
  * GtkSourceView is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,13 +17,10 @@
  * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
 #include "gtksourcegutterrendererpixbuf.h"
-#include "gtksourceview-i18n.h"
-#include "gtksourcepixbufhelper.h"
+#include "gtksourcepixbufhelper-private.h"
 
 /**
  * SECTION:gutterrendererpixbuf
@@ -36,10 +32,12 @@
  * #GtkSourceGutter.
  */
 
-struct _GtkSourceGutterRendererPixbufPrivate
+typedef struct
 {
 	GtkSourcePixbufHelper *helper;
-};
+	GdkPaintable          *paintable;
+	GPtrArray             *overlays;
+} GtkSourceGutterRendererPixbufPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceGutterRendererPixbuf, gtk_source_gutter_renderer_pixbuf, GTK_SOURCE_TYPE_GUTTER_RENDERER)
 
@@ -49,154 +47,90 @@ enum
 	PROP_PIXBUF,
 	PROP_ICON_NAME,
 	PROP_GICON,
+        PROP_PAINTABLE,
 };
 
 static void
-center_on (GtkSourceGutterRenderer *renderer,
-           GdkRectangle            *cell_area,
-           GtkTextIter             *iter,
-           gint                     width,
-           gint                     height,
-           gfloat                   xalign,
-           gfloat                   yalign,
-           gint                    *x,
-           gint                    *y)
+clear_overlays (GtkSourceGutterRendererPixbuf *renderer)
 {
-	GtkTextView *view;
-	GtkTextWindowType window_type;
-	GdkRectangle buffer_location;
-	gint window_y;
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
 
-	view = gtk_source_gutter_renderer_get_view (renderer);
-	window_type = gtk_source_gutter_renderer_get_window_type (renderer);
-
-	gtk_text_view_get_iter_location (view, iter, &buffer_location);
-
-	gtk_text_view_buffer_to_window_coords (view,
-					       window_type,
-					       0, buffer_location.y,
-					       NULL, &window_y);
-
-	*x = cell_area->x + (cell_area->width - width) * xalign;
-	*y = window_y + (buffer_location.height - height) * yalign;
+	if (priv->overlays != NULL && priv->overlays->len > 0)
+	{
+		g_ptr_array_remove_range (priv->overlays, 0, priv->overlays->len);
+	}
 }
 
 static void
-gutter_renderer_pixbuf_draw (GtkSourceGutterRenderer      *renderer,
-                             cairo_t                      *cr,
-                             GdkRectangle                 *background_area,
-                             GdkRectangle                 *cell_area,
-                             GtkTextIter                  *start,
-                             GtkTextIter                  *end,
-                             GtkSourceGutterRendererState  state)
+gutter_renderer_pixbuf_snapshot_line (GtkSourceGutterRenderer      *renderer,
+                                      GtkSnapshot                  *snapshot,
+                                      GtkSourceGutterLines         *lines,
+                                      guint                         line)
 {
 	GtkSourceGutterRendererPixbuf *pix = GTK_SOURCE_GUTTER_RENDERER_PIXBUF (renderer);
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (pix);
+	GtkWidget *widget = GTK_WIDGET (renderer);
+	GdkPaintable *paintable;
+	GtkSourceView *view;
 	gint width;
 	gint height;
-	gfloat xalign;
-	gfloat yalign;
-	GtkSourceGutterRendererAlignmentMode mode;
-	GtkTextView *view;
-	gint scale;
-	gint x = 0;
-	gint y = 0;
-	GdkPixbuf *pixbuf;
-	cairo_surface_t *surface;
-
-	/* Chain up to draw background */
-	if (GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_pixbuf_parent_class)->draw != NULL)
-	{
-		GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_pixbuf_parent_class)->draw (renderer,
-													 cr,
-													 background_area,
-													 cell_area,
-													 start,
-													 end,
-													 state);
-	}
+	gfloat x = 0;
+	gfloat y = 0;
+	guint i;
+	gint size;
 
 	view = gtk_source_gutter_renderer_get_view (renderer);
 
-	pixbuf = gtk_source_pixbuf_helper_render (pix->priv->helper,
-	                                          GTK_WIDGET (view),
-	                                          cell_area->width);
+	width = gtk_widget_get_width (widget);
+	height = gtk_widget_get_height (widget);
+	size = MIN (width, height);
 
-	if (!pixbuf)
+	paintable = gtk_source_pixbuf_helper_render (priv->helper,
+	                                             GTK_WIDGET (view),
+	                                             size);
+
+	/* Short-circuit if there is nothing to snapshot */
+	if (paintable == NULL &&
+	    (priv->overlays == NULL || priv->overlays->len == 0))
 	{
 		return;
 	}
 
-	width = gdk_pixbuf_get_width (pixbuf);
-	height = gdk_pixbuf_get_height (pixbuf);
+	gtk_source_gutter_renderer_align_cell (renderer, line, size, size, &x, &y);
 
-	/*
-	 * We might have gotten a pixbuf back from the helper that will allow
-	 * us to render for HiDPI. If we detect this, we pretend that we got a
-	 * different size back and then gdk_cairo_surface_create_from_pixbuf()
-	 * will take care of the rest.
-	 */
-	scale = gtk_widget_get_scale_factor (GTK_WIDGET (view));
-	if ((scale > 1) &&
-	    ((width > cell_area->width) || (height > cell_area->height)) &&
-	    (width <= (cell_area->width * scale)) &&
-	    (height <= (cell_area->height * scale)))
+	gtk_snapshot_save (snapshot);
+	gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT (x, y));
+
+	if (paintable != NULL)
 	{
-		width = width / scale;
-		height = height / scale;
+		gdk_paintable_snapshot (paintable, snapshot, size, size);
+	}
+	else if (priv->paintable != NULL)
+	{
+		gdk_paintable_snapshot (priv->paintable, snapshot, size, size);
 	}
 
-	gtk_source_gutter_renderer_get_alignment (renderer,
-	                                          &xalign,
-	                                          &yalign);
-
-	mode = gtk_source_gutter_renderer_get_alignment_mode (renderer);
-
-	switch (mode)
+	if (priv->overlays != NULL)
 	{
-		case GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_CELL:
-			x = cell_area->x + (cell_area->width - width) * xalign;
-			y = cell_area->y + (cell_area->height - height) * yalign;
-			break;
-		case GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_FIRST:
-			center_on (renderer,
-			           cell_area,
-			           start,
-			           width,
-			           height,
-			           xalign,
-			           yalign,
-			           &x,
-			           &y);
-			break;
-		case GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_LAST:
-			center_on (renderer,
-			           cell_area,
-			           end,
-			           width,
-			           height,
-			           xalign,
-			           yalign,
-			           &x,
-			           &y);
-			break;
-		default:
-			g_assert_not_reached ();
+		for (i = 0; i < priv->overlays->len; i++)
+		{
+			GdkPaintable *ele = g_ptr_array_index (priv->overlays, i);
+
+			gdk_paintable_snapshot (ele, snapshot, size, size);
+		}
 	}
 
-	surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale, NULL);
-	cairo_set_source_surface (cr, surface, x, y);
-
-	cairo_paint (cr);
-
-	cairo_surface_destroy (surface);
+	gtk_snapshot_restore (snapshot);
 }
 
 static void
 gtk_source_gutter_renderer_pixbuf_finalize (GObject *object)
 {
 	GtkSourceGutterRendererPixbuf *renderer = GTK_SOURCE_GUTTER_RENDERER_PIXBUF (object);
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
 
-	gtk_source_pixbuf_helper_free (renderer->priv->helper);
+        g_clear_pointer (&priv->helper, gtk_source_pixbuf_helper_free);
+        g_clear_pointer (&priv->overlays, g_ptr_array_unref);
 
 	G_OBJECT_CLASS (gtk_source_gutter_renderer_pixbuf_parent_class)->finalize (object);
 }
@@ -205,36 +139,33 @@ static void
 set_pixbuf (GtkSourceGutterRendererPixbuf *renderer,
             GdkPixbuf                     *pixbuf)
 {
-	gtk_source_pixbuf_helper_set_pixbuf (renderer->priv->helper,
-	                                     pixbuf);
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
 
-	g_object_notify (G_OBJECT (renderer), "pixbuf");
-
-	gtk_source_gutter_renderer_queue_draw (GTK_SOURCE_GUTTER_RENDERER (renderer));
+        g_clear_object (&priv->paintable);
+        clear_overlays (renderer);
+	gtk_source_pixbuf_helper_set_pixbuf (priv->helper, pixbuf);
 }
 
 static void
 set_gicon (GtkSourceGutterRendererPixbuf *renderer,
            GIcon                         *icon)
 {
-	gtk_source_pixbuf_helper_set_gicon (renderer->priv->helper,
-	                                    icon);
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
 
-	g_object_notify (G_OBJECT (renderer), "gicon");
-
-	gtk_source_gutter_renderer_queue_draw (GTK_SOURCE_GUTTER_RENDERER (renderer));
+        g_clear_object (&priv->paintable);
+        clear_overlays (renderer);
+	gtk_source_pixbuf_helper_set_gicon (priv->helper, icon);
 }
 
 static void
 set_icon_name (GtkSourceGutterRendererPixbuf *renderer,
                const gchar                   *icon_name)
 {
-	gtk_source_pixbuf_helper_set_icon_name (renderer->priv->helper,
-	                                        icon_name);
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
 
-	g_object_notify (G_OBJECT (renderer), "icon-name");
-
-	gtk_source_gutter_renderer_queue_draw (GTK_SOURCE_GUTTER_RENDERER (renderer));
+        g_clear_object (&priv->paintable);
+        clear_overlays (renderer);
+	gtk_source_pixbuf_helper_set_icon_name (priv->helper, icon_name);
 }
 
 
@@ -244,9 +175,7 @@ gtk_source_gutter_renderer_pixbuf_set_property (GObject      *object,
                                                 const GValue *value,
                                                 GParamSpec   *pspec)
 {
-	GtkSourceGutterRendererPixbuf *renderer;
-
-	renderer = GTK_SOURCE_GUTTER_RENDERER_PIXBUF (object);
+	GtkSourceGutterRendererPixbuf *renderer = GTK_SOURCE_GUTTER_RENDERER_PIXBUF (object);
 
 	switch (prop_id)
 	{
@@ -259,6 +188,10 @@ gtk_source_gutter_renderer_pixbuf_set_property (GObject      *object,
 		case PROP_GICON:
 			set_gicon (renderer, g_value_get_object (value));
 			break;
+		case PROP_PAINTABLE:
+			gtk_source_gutter_renderer_pixbuf_set_paintable (renderer,
+			                                                 g_value_get_object (value));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -271,23 +204,25 @@ gtk_source_gutter_renderer_pixbuf_get_property (GObject    *object,
                                                 GValue     *value,
                                                 GParamSpec *pspec)
 {
-	GtkSourceGutterRendererPixbuf *renderer;
-
-	renderer = GTK_SOURCE_GUTTER_RENDERER_PIXBUF (object);
+	GtkSourceGutterRendererPixbuf *renderer = GTK_SOURCE_GUTTER_RENDERER_PIXBUF (object);
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
 
 	switch (prop_id)
 	{
 		case PROP_PIXBUF:
 			g_value_set_object (value,
-			                    gtk_source_pixbuf_helper_get_pixbuf (renderer->priv->helper));
+			                    gtk_source_pixbuf_helper_get_pixbuf (priv->helper));
 			break;
 		case PROP_ICON_NAME:
 			g_value_set_string (value,
-			                    gtk_source_pixbuf_helper_get_icon_name (renderer->priv->helper));
+			                    gtk_source_pixbuf_helper_get_icon_name (priv->helper));
 			break;
 		case PROP_GICON:
 			g_value_set_object (value,
-			                    gtk_source_pixbuf_helper_get_gicon (renderer->priv->helper));
+			                    gtk_source_pixbuf_helper_get_gicon (priv->helper));
+			break;
+		case PROP_PAINTABLE:
+			g_value_set_object (value, priv->paintable);
 			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -302,11 +237,18 @@ gtk_source_gutter_renderer_pixbuf_class_init (GtkSourceGutterRendererPixbufClass
 	GtkSourceGutterRendererClass *renderer_class = GTK_SOURCE_GUTTER_RENDERER_CLASS (klass);
 
 	object_class->finalize = gtk_source_gutter_renderer_pixbuf_finalize;
-
 	object_class->get_property = gtk_source_gutter_renderer_pixbuf_get_property;
 	object_class->set_property = gtk_source_gutter_renderer_pixbuf_set_property;
 
-	renderer_class->draw = gutter_renderer_pixbuf_draw;
+	renderer_class->snapshot_line = gutter_renderer_pixbuf_snapshot_line;
+
+	g_object_class_install_property (object_class,
+	                                 PROP_PAINTABLE,
+	                                 g_param_spec_object ("paintable",
+	                                                      "Paintable",
+	                                                      "The paintable",
+	                                                      GDK_TYPE_PAINTABLE,
+	                                                      G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
 
 	g_object_class_install_property (object_class,
 	                                 PROP_PIXBUF,
@@ -314,7 +256,7 @@ gtk_source_gutter_renderer_pixbuf_class_init (GtkSourceGutterRendererPixbufClass
 	                                                      "Pixbuf",
 	                                                      "The pixbuf",
 	                                                      GDK_TYPE_PIXBUF,
-	                                                      G_PARAM_READWRITE));
+	                                                      G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
 
 	g_object_class_install_property (object_class,
 	                                 PROP_ICON_NAME,
@@ -322,7 +264,7 @@ gtk_source_gutter_renderer_pixbuf_class_init (GtkSourceGutterRendererPixbufClass
 	                                                      "Icon Name",
 	                                                      "The icon name",
 	                                                      NULL,
-	                                                      G_PARAM_READWRITE));
+	                                                      G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
 
 	g_object_class_install_property (object_class,
 	                                 PROP_GICON,
@@ -330,15 +272,15 @@ gtk_source_gutter_renderer_pixbuf_class_init (GtkSourceGutterRendererPixbufClass
 	                                                      "GIcon",
 	                                                      "The gicon",
 	                                                      G_TYPE_ICON,
-	                                                      G_PARAM_READWRITE));
+	                                                      G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
 }
 
 static void
 gtk_source_gutter_renderer_pixbuf_init (GtkSourceGutterRendererPixbuf *self)
 {
-	self->priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (self);
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (self);
 
-	self->priv->helper = gtk_source_pixbuf_helper_new ();
+	priv->helper = gtk_source_pixbuf_helper_new ();
 }
 
 /**
@@ -383,9 +325,11 @@ gtk_source_gutter_renderer_pixbuf_set_pixbuf (GtkSourceGutterRendererPixbuf *ren
 GdkPixbuf *
 gtk_source_gutter_renderer_pixbuf_get_pixbuf (GtkSourceGutterRendererPixbuf *renderer)
 {
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
+
 	g_return_val_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER_PIXBUF (renderer), NULL);
 
-	return gtk_source_pixbuf_helper_get_pixbuf (renderer->priv->helper);
+	return gtk_source_pixbuf_helper_get_pixbuf (priv->helper);
 }
 
 /**
@@ -415,9 +359,11 @@ gtk_source_gutter_renderer_pixbuf_set_gicon (GtkSourceGutterRendererPixbuf *rend
 GIcon *
 gtk_source_gutter_renderer_pixbuf_get_gicon (GtkSourceGutterRendererPixbuf *renderer)
 {
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
+
 	g_return_val_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER_PIXBUF (renderer), NULL);
 
-	return gtk_source_pixbuf_helper_get_gicon (renderer->priv->helper);
+	return gtk_source_pixbuf_helper_get_gicon (priv->helper);
 }
 
 /**
@@ -437,7 +383,77 @@ gtk_source_gutter_renderer_pixbuf_set_icon_name (GtkSourceGutterRendererPixbuf *
 const gchar *
 gtk_source_gutter_renderer_pixbuf_get_icon_name (GtkSourceGutterRendererPixbuf *renderer)
 {
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
+
 	g_return_val_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER_PIXBUF (renderer), NULL);
 
-	return gtk_source_pixbuf_helper_get_icon_name (renderer->priv->helper);
+	return gtk_source_pixbuf_helper_get_icon_name (priv->helper);
+}
+
+/**
+ * gtk_source_gutter_renderer_pixbuf_set_paintable:
+ * @renderer: a #GtkSourceGutterRendererPixbuf
+ * @paintable: (nullable): the paintable, or %NULL.
+ */
+void
+gtk_source_gutter_renderer_pixbuf_set_paintable (GtkSourceGutterRendererPixbuf *renderer,
+                                                 GdkPaintable                  *paintable)
+{
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
+
+	g_return_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER_PIXBUF (renderer));
+	g_return_if_fail (!paintable || GDK_IS_PAINTABLE (paintable));
+
+	clear_overlays (renderer);
+	gtk_source_pixbuf_helper_set_icon_name (priv->helper, NULL);
+	g_set_object (&priv->paintable, paintable);
+}
+
+/**
+ * gtk_source_gutter_renderer_pixbuf_get_paintable:
+ * @renderer: a #GtkSourceGutterRendererPixbuf
+ *
+ * Gets a #GdkPaintable that was set with
+ * gtk_source_gutter_renderer_pixbuf_set_paintable()
+ *
+ * Returns: (transfer none) (nullable): a #GdkPaintable or %NULL
+ *
+ * Since: 5.0
+ */
+GdkPaintable *
+gtk_source_gutter_renderer_pixbuf_get_paintable (GtkSourceGutterRendererPixbuf *renderer)
+{
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
+
+	g_return_val_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER_PIXBUF (renderer), NULL);
+
+	return priv->paintable;
+}
+
+/**
+ * gtk_source_gutter_renderer_pixbuf_overlay_paintable:
+ * @renderer: a #GtkSourceGutterRendererPixbuf
+ * @paintable: a #GdkPaintable
+ *
+ * Allows overlaying a paintable on top of any other image that
+ * has been set for the pixbuf. This will be applied when the
+ * widget is next snapshot.
+ *
+ * Since: 5.0
+ */
+void
+gtk_source_gutter_renderer_pixbuf_overlay_paintable (GtkSourceGutterRendererPixbuf *renderer,
+                                                     GdkPaintable                  *paintable)
+{
+	GtkSourceGutterRendererPixbufPrivate *priv = gtk_source_gutter_renderer_pixbuf_get_instance_private (renderer);
+
+	g_return_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER_PIXBUF (renderer));
+	g_return_if_fail (GDK_IS_PAINTABLE (paintable));
+
+	if (priv->overlays == NULL)
+	{
+		priv->overlays = g_ptr_array_new_with_free_func (g_object_unref);
+	}
+
+	g_ptr_array_add (priv->overlays, g_object_ref (paintable));
 }
