@@ -18,6 +18,7 @@
 
 #include "config.h"
 
+#include <math.h>
 #include <string.h>
 
 #if ENABLE_FONT_CONFIG
@@ -34,6 +35,7 @@
 #include "gtksourceutils-private.h"
 
 #define SCRUBBER_MIN_HEIGHT 10
+#define DRAG_THRESHOLD      5.0
 
 /**
  * SECTION:map
@@ -96,9 +98,9 @@
  * pixel cache when cairo_scale is not 1-to-1. This results in layout
  * invalidation and worst case render paths.
  *
- * I also tried rendering the scrubber (overlay box) during the
+ * I also tried rendering the slider (overlay box) during the
  * GtkTextView::draw_layer() vfunc. The problem with this approach is that
- * the scrubber contents are actually pixel cached. So every time the scrubber
+ * the slider contents are actually pixel cached. So every time the slider
  * moves we have to invalidate the GtkTextLayout and redraw cached contents.
  * Where as drawing in the GtkTextView::draw() vfunc, after the pixel cache
  * contents have been drawn results in only a composite blend, not
@@ -162,7 +164,7 @@ typedef struct
 	PangoFontDescription *font_desc;
 
 	/*
-	 * The easiest way to style the scrubber and the sourceview is
+	 * The easiest way to style the slider and the sourceview is
 	 * by using CSS. This is necessary since we can't mess with the
 	 * fonts used in the textbuffer (as one might using GtkTextTag).
 	 */
@@ -177,8 +179,8 @@ typedef struct
 	/* The slider widget */
 	GtkSourceMapSlider *slider;
 
-	/* The location of the scrubber in widget coordinate space. */
-	GdkRectangle scrubber_area;
+	/* The location of the slider in widget coordinate space. */
+	GdkRectangle slider_area;
 
 	/* Weak pointer view to child view bindings */
 	GBinding *buffer_binding;
@@ -196,11 +198,14 @@ typedef struct
 	/* Denotes if we are in a grab from button press */
 	guint in_press : 1;
 
-	/* If we failed to locate a color for the scrubber, then this will
+	/* If we failed to locate a color for the slider, then this will
 	 * be set to 0 and that means we need to apply the "selection"
 	 * class when drawing so that we get an appropriate color.
 	 */
 	guint had_color : 1;
+
+	/* If we dragged enough to reach a drag threshold */
+	guint reached_drag_threshold : 1;
 } GtkSourceMapPrivate;
 
 enum
@@ -240,7 +245,7 @@ load_override_font (GtkSourceMap *map)
 	font_map = pango_cairo_font_map_new_for_font_type (CAIRO_FONT_TYPE_FT);
 	pango_fc_font_map_set_config (PANGO_FC_FONT_MAP (font_map), map_font_config);
 	gtk_widget_set_font_map (GTK_WIDGET (map), font_map);
-	font_desc = pango_font_description_from_string ("Builder Blocks 1");
+	font_desc = pango_font_description_from_string ("BuilderBlocks 2");
 
 	g_assert (map_font_config != NULL);
 	g_assert (font_map != NULL);
@@ -254,68 +259,55 @@ load_override_font (GtkSourceMap *map)
 #endif
 
 static void
-update_scrubber_position (GtkSourceMap *map)
+update_slider_position (GtkSourceMap *map)
 {
-	GtkSourceMapPrivate *priv;
-	GtkTextIter iter;
-	GdkRectangle visible_area;
-	GdkRectangle iter_area;
-	GdkRectangle scrubber_area;
-	GtkAllocation alloc;
-	GtkAllocation view_alloc;
-	gint ignored;
-	gint child_height;
-	gint view_height;
-
-	priv = gtk_source_map_get_instance_private (map);
+	GtkSourceMapPrivate *priv = gtk_source_map_get_instance_private (map);
+	GdkRectangle them_visible_rect;
+	GdkRectangle us_alloc;
+	GdkRectangle slider_area;
+	GtkStyleContext *style_context;
+	GtkTextBuffer *buffer;
+	GtkTextIter end_iter;
+	GdkRectangle end_rect;
+	GtkBorder border;
+	int us_height;
+	int them_height;
 
 	if (priv->view == NULL)
 	{
 		return;
 	}
 
-	gtk_widget_get_allocation (GTK_WIDGET (priv->view), &view_alloc);
-	gtk_widget_get_allocation (GTK_WIDGET (map), &alloc);
+	gtk_widget_get_allocation (GTK_WIDGET (map), &us_alloc);
 
-	gtk_widget_measure (GTK_WIDGET (priv->view),
-	                    GTK_ORIENTATION_VERTICAL,
-	                    gtk_widget_get_width (GTK_WIDGET (priv->view)),
-	                    NULL,
-	                    &view_height,
-	                    NULL, NULL);
-	GTK_WIDGET_CLASS (gtk_source_map_parent_class)->measure (GTK_WIDGET (map),
-								 GTK_ORIENTATION_VERTICAL,
-								 gtk_widget_get_width (GTK_WIDGET (map)),
-								 &ignored, &child_height, &ignored, &ignored);
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (map));
+	style_context = gtk_widget_get_style_context (GTK_WIDGET (map));
+	gtk_style_context_get_border (style_context, &border);
 
-	gtk_text_view_get_visible_rect (GTK_TEXT_VIEW (priv->view), &visible_area);
-	gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (priv->view), &iter,
-	                                    visible_area.x, visible_area.y);
-	gtk_text_view_get_iter_location (GTK_TEXT_VIEW (map), &iter, &iter_area);
+	gtk_text_buffer_get_end_iter (buffer, &end_iter);
+	gtk_text_view_get_iter_location (GTK_TEXT_VIEW (map), &end_iter, &end_rect);
+	us_height = end_rect.y + end_rect.height;
+	gtk_text_view_get_iter_location (GTK_TEXT_VIEW (priv->view), &end_iter, &end_rect);
+	them_height = end_rect.y + end_rect.height;
 
-	scrubber_area.x = 0;
-	scrubber_area.width = alloc.width;
-	scrubber_area.y = iter_area.y;
-	scrubber_area.height = ((gdouble)view_alloc.height /
-	                        (gdouble)view_height *
-	                        (gdouble)child_height) +
-	                       iter_area.height;
+	gtk_text_view_get_visible_rect (GTK_TEXT_VIEW (priv->view), &them_visible_rect);
 
-	if (scrubber_area.height > alloc.height)
+	slider_area.x = 0;
+	slider_area.width = us_alloc.width - border.left - border.right;
+	slider_area.height = 100;
+	slider_area.y = (double)them_visible_rect.y / (double)them_height * (double)us_height;
+	slider_area.height = ((double)(them_visible_rect.y + them_visible_rect.height) / (double)them_height * (double)us_height) - slider_area.y;
+
+	if (memcmp (&slider_area, &priv->slider_area, sizeof slider_area) != 0)
 	{
-		scrubber_area.height = alloc.height;
-	}
-
-	if (memcmp (&scrubber_area, &priv->scrubber_area, sizeof scrubber_area) != 0)
-	{
-		priv->scrubber_area = scrubber_area;
+		priv->slider_area = slider_area;
 		gtk_widget_set_size_request (GTK_WIDGET (priv->slider),
-		                             scrubber_area.width,
-		                             scrubber_area.height);
+		                             MAX (0, slider_area.width),
+		                             MAX (0, slider_area.height));
 		gtk_text_view_move_overlay (GTK_TEXT_VIEW (map),
 		                            GTK_WIDGET (priv->slider),
-					    scrubber_area.x,
-		                            scrubber_area.y);
+		                            slider_area.x,
+		                            slider_area.y);
 		gtk_widget_queue_allocate (GTK_WIDGET (map));
 	}
 }
@@ -328,7 +320,8 @@ gtk_source_map_rebuild_css (GtkSourceMap *map)
 	GtkSourceStyle *style = NULL;
 	GtkTextBuffer *buffer;
 	GString *gstr;
-	gchar *background = NULL;
+	char *background = NULL;
+	char *foreground = NULL;
 
 	priv = gtk_source_map_get_instance_private (map);
 
@@ -339,17 +332,17 @@ gtk_source_map_rebuild_css (GtkSourceMap *map)
 
 	/*
 	 * This is where we calculate the CSS that maps the font for the
-	 * minimap as well as the styling for the scrubber.
+	 * minimap as well as the styling for the slider.
 	 *
 	 * The font is calculated from #GtkSourceMap:font-desc. We convert this
 	 * to CSS using _gtk_source_utils_pango_font_description_to_css(). It
 	 * gets applied to the minimap widget via the CSS style provider which
 	 * we attach to the view in gtk_source_map_init().
 	 *
-	 * The rules for calculating the style for the scrubber are as follows.
+	 * The rules for calculating the style for the slider are as follows.
 	 *
 	 * If the current style scheme provides a background color for the
-	 * scrubber using the "map-overlay" style name, we use that without
+	 * slider using the "map-overlay" style name, we use that without
 	 * any transformations.
 	 *
 	 * If the style scheme contains a "selection" style scheme, used for
@@ -390,36 +383,66 @@ gtk_source_map_rebuild_css (GtkSourceMap *map)
 
 	if (style != NULL)
 	{
+		gboolean foreground_set;
+		gboolean background_set;
+
 		g_object_get (style,
+		              "foreground", &foreground,
+		              "foreground-set", &foreground_set,
 		              "background", &background,
+		              "background-set", &background_set,
 		              NULL);
+
+		if (!foreground_set)
+		{
+			g_clear_pointer (&foreground, g_free);
+		}
+
+		if (!background_set)
+		{
+			g_clear_pointer (&background, g_free);
+		}
 	}
 
 	priv->had_color = background != NULL;
 
 	if (background != NULL)
 	{
+		if (foreground == NULL)
+		{
+			GtkStyleContext *style_context;
+			GdkRGBA color;
+
+			style_context = gtk_widget_get_style_context (GTK_WIDGET (map));
+			gtk_style_context_get_color (style_context, &color);
+			foreground = gdk_rgba_to_string (&color);
+		}
+
 		g_string_append_printf (gstr,
-		                        "slider {\n"
-		                        "\tbackground-color: alpha(%s,.3);\n"
-		                        "\tborder-radius: 3px;\n"
-		                        "\tborder: 1px solid alpha(shade(%s,.9),.3);\n"
-		                        "\tmargin-top: 1px;\n"
-		                        "\tmargin-bottom: 1px;\n"
+		                        "textview.source-map {"
+		                        " color: mix(%s,%s,.25);"
 		                        "}\n"
-		                        "slider:hover {\n"
-		                        "\tbackground-color: alpha(%s,.5);\n"
+		                        "slider {"
+		                        " background-color: alpha(%s,.25);"
+		                        " transition-duration: 300ms;"
+		                        "}\n"
+		                        "slider:hover {"
+		                        " background-color: alpha(%s,.35);"
+		                        "}\n"
+		                        "slider.dragging:hover {"
+		                        " background-color: alpha(%s,.5);"
 		                        "}\n",
-		                        background,
-		                        background,
-		                        background);
+					foreground, background,
+		                        background, background, background);
 	}
 
 	g_free (background);
+	g_free (foreground);
 
 	if (gstr->len > 0)
 	{
-		gtk_css_provider_load_from_data (priv->css_provider, gstr->str, gstr->len);
+		/* use -1 to avoid a copy of the string */
+		gtk_css_provider_load_from_data (priv->css_provider, gstr->str, -1);
 	}
 
 	g_string_free (gstr, TRUE);
@@ -471,7 +494,7 @@ view_vadj_value_changed (GtkSourceMap  *map,
                          GtkAdjustment *vadj)
 {
 	update_child_vadjustment (map);
-	update_scrubber_position (map);
+	update_slider_position (map);
 }
 
 static void
@@ -479,7 +502,7 @@ view_vadj_notify_upper (GtkSourceMap  *map,
                         GParamSpec    *pspec,
                         GtkAdjustment *vadj)
 {
-	update_scrubber_position (map);
+	update_slider_position (map);
 }
 
 static void
@@ -617,8 +640,16 @@ gtk_source_map_measure (GtkWidget      *widget,
 		else
 		{
 			PangoLayout *layout;
-			gint height;
-			gint width;
+			char *text;
+			guint right_margin_position;
+			int height;
+			int width;
+
+			right_margin_position = gtk_source_view_get_right_margin_position (priv->view);
+
+			text = g_malloc (right_margin_position + 1);
+			memset (text, 'X', right_margin_position);
+			text[right_margin_position] = 0;
 
 			/*
 			 * FIXME:
@@ -627,11 +658,15 @@ gtk_source_map_measure (GtkWidget      *widget,
 			 * rebuilding our CSS since it gets used a bunch and changes
 			 * very little.
 			 */
-			layout = gtk_widget_create_pango_layout (GTK_WIDGET (map), "X");
+			layout = gtk_widget_create_pango_layout (GTK_WIDGET (map), text);
 			pango_layout_get_pixel_size (layout, &width, &height);
 			g_object_unref (layout);
+			g_free (text);
 
-			width *= gtk_source_view_get_right_margin_position (priv->view);
+			/* If left-margin is set, try to balance the right side with the same amount
+			 * of additional space to keep it aligned.
+			 */
+			width += (gtk_text_view_get_left_margin (GTK_TEXT_VIEW (map)) * 2);
 
 			*minimum = *natural = width;
 		}
@@ -670,8 +705,8 @@ scroll_to_child_point (GtkSourceMap *map,
 
 		gtk_widget_get_allocation (GTK_WIDGET (map), &alloc);
 		gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (map), &iter, x, y);
-		gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (priv->view), &iter,
-		                              0.0, TRUE, 1.0, 0.5);
+		_gtk_source_view_jump_to_iter (GTK_TEXT_VIEW (priv->view), &iter,
+		                               0.0, TRUE, 1.0, 0.5);
 	}
 }
 
@@ -880,24 +915,32 @@ gtk_source_map_set_property (GObject      *object,
 
 static void
 gtk_source_map_drag_update (GtkSourceMap   *map,
-			    gdouble         x,
-			    gdouble         y,
-			    GtkGestureDrag *drag)
+                            double          x,
+                            double          y,
+                            GtkGestureDrag *drag)
 {
+	GtkSourceMapPrivate *priv = gtk_source_map_get_instance_private (map);
 	GtkTextBuffer *buffer;
 	GtkAllocation alloc;
 	GdkRectangle area;
 	GtkTextIter iter;
-	gdouble yratio;
-	gdouble begin_x;
-	gdouble begin_y;
-	gint ignored;
-	gint real_height;
-	gint height;
+	double yratio;
+	double begin_x;
+	double begin_y;
+	int ignored;
+	int real_height;
+	int height;
+
+	if (!priv->reached_drag_threshold && ABS (y) < DRAG_THRESHOLD)
+	{
+		return;
+	}
+
+	priv->reached_drag_threshold = TRUE;
 
 	gtk_widget_get_allocation (GTK_WIDGET (map), &alloc);
 	gtk_gesture_drag_get_start_point (drag, &begin_x, &begin_y);
-	y = CLAMP (begin_y + y, 0, alloc.height);
+	y = CLAMP (ceil (begin_y + y), 0, alloc.height);
 
 	GTK_WIDGET_CLASS (gtk_source_map_parent_class)->measure (GTK_WIDGET (map),
 								 GTK_ORIENTATION_VERTICAL,
@@ -910,21 +953,66 @@ gtk_source_map_drag_update (GtkSourceMap   *map,
 	gtk_text_buffer_get_end_iter (buffer, &iter);
 	gtk_text_view_get_iter_location (GTK_TEXT_VIEW (map), &iter, &area);
 
-	yratio = CLAMP (y, 0, height) / (gdouble)height;
+	yratio = y / (double)height;
 
 	scroll_to_child_point (map, 0, real_height * yratio);
 }
 
 static void
 gtk_source_map_drag_begin (GtkSourceMap   *map,
-			   gdouble         start_x,
-			   gdouble         start_y,
-			   GtkGestureDrag *drag)
+                           double          start_x,
+                           double          start_y,
+                           GtkGestureDrag *drag)
 {
+	GtkSourceMapPrivate *priv = gtk_source_map_get_instance_private (map);
+
+	priv->reached_drag_threshold = FALSE;
+
 	gtk_gesture_set_state (GTK_GESTURE (drag), GTK_EVENT_SEQUENCE_CLAIMED);
 	gtk_source_map_drag_update (map, 0, 0, drag);
+
+	gtk_widget_add_css_class (GTK_WIDGET (priv->slider), "dragging");
 }
 
+static void
+gtk_source_map_drag_end (GtkSourceMap   *map,
+                         double          start_x,
+                         double          start_y,
+                         GtkGestureDrag *drag)
+{
+	GtkSourceMapPrivate *priv = gtk_source_map_get_instance_private (map);
+
+	gtk_widget_remove_css_class (GTK_WIDGET (priv->slider), "dragging");
+}
+
+static void
+gtk_source_map_click_pressed (GtkSourceMap *map,
+                              int           n_presses,
+                              double        x,
+                              double        y,
+                              GtkGesture   *click)
+{
+	GtkSourceMapPrivate *priv = gtk_source_map_get_instance_private (map);
+	GtkTextIter iter;
+	int buffer_x, buffer_y;
+
+	g_assert (GTK_SOURCE_IS_MAP (map));
+	g_assert (GTK_IS_GESTURE_CLICK (click));
+
+	if (priv->view == NULL)
+	{
+		return;
+	}
+
+	gtk_gesture_set_state (click, GTK_EVENT_SEQUENCE_CLAIMED);
+
+	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (map),
+	                                       GTK_TEXT_WINDOW_WIDGET,
+	                                       x, y, &buffer_x, &buffer_y);
+	gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (map), &iter, 0, buffer_y);
+	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (priv->view), &iter,
+	                              0.0, TRUE, 1.0, 0.5);
+}
 
 static gboolean
 gtk_source_map_scroll (GtkWidget *widget,
@@ -1025,6 +1113,32 @@ gtk_source_map_hide (GtkWidget *widget)
 	}
 }
 
+static void
+gtk_source_map_css_changed (GtkWidget         *widget,
+                            GtkCssStyleChange *change)
+{
+	g_assert (GTK_IS_WIDGET (widget));
+
+	GTK_WIDGET_CLASS (gtk_source_map_parent_class)->css_changed (widget, change);
+
+#if GTK_CHECK_VERSION(4,3,1)
+	{
+		PangoContext *rtl_context;
+		PangoContext *ltr_context;
+
+		/* Ensure rounding so that BuilderBlocks aligns properly within
+		 * rounding errors between glyphs.
+		 */
+
+		rtl_context = gtk_text_view_get_rtl_context (GTK_TEXT_VIEW (widget));
+		ltr_context = gtk_text_view_get_ltr_context (GTK_TEXT_VIEW (widget));
+
+		pango_context_set_round_glyph_positions (rtl_context, TRUE);
+		pango_context_set_round_glyph_positions (ltr_context, TRUE);
+	}
+#endif
+}
+
 static void
 gtk_source_map_constructed (GObject *object)
 {
@@ -1052,6 +1166,7 @@ gtk_source_map_class_init (GtkSourceMapClass *klass)
 	widget_class->show = gtk_source_map_show;
 	widget_class->state_flags_changed = gtk_source_map_state_flags_changed;
 	widget_class->realize = gtk_source_map_realize;
+	widget_class->css_changed = gtk_source_map_css_changed;
 
 	text_view_class->snapshot_layer = gtk_source_map_snapshot_layer;
 
@@ -1078,6 +1193,7 @@ gtk_source_map_init (GtkSourceMap *map)
 	GtkSourceMapPrivate *priv;
 	GtkSourceCompletion *completion;
 	GtkEventController *scroll;
+	GtkEventController *press;
 	GtkGesture *drag;
 
 	priv = gtk_source_map_get_instance_private (map);
@@ -1110,19 +1226,23 @@ gtk_source_map_init (GtkSourceMap *map)
 	completion = gtk_source_view_get_completion (GTK_SOURCE_VIEW (map));
 	gtk_source_completion_block_interactive (completion);
 
-	gtk_source_map_set_font_name (map, "BuilderBlocks 1");
+	gtk_source_map_set_font_name (map, "BuilderBlocks 2");
 
 	drag = gtk_gesture_drag_new ();
 	gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (drag),
                                                     GTK_PHASE_CAPTURE);
 	g_signal_connect_swapped (drag,
-				  "drag-begin",
-				  G_CALLBACK (gtk_source_map_drag_begin),
-				  map);
+	                          "drag-begin",
+	                          G_CALLBACK (gtk_source_map_drag_begin),
+	                          map);
 	g_signal_connect_swapped (drag,
-				  "drag-update",
-				  G_CALLBACK (gtk_source_map_drag_update),
-				  map);
+	                          "drag-end",
+	                          G_CALLBACK (gtk_source_map_drag_end),
+	                          map);
+	g_signal_connect_swapped (drag,
+	                          "drag-update",
+	                          G_CALLBACK (gtk_source_map_drag_update),
+	                          map);
 	gtk_widget_add_controller (GTK_WIDGET (map),
 	                           GTK_EVENT_CONTROLLER (g_steal_pointer (&drag)));
 
@@ -1133,6 +1253,14 @@ gtk_source_map_init (GtkSourceMap *map)
 	                          G_CALLBACK (gtk_source_map_scroll),
 	                          map);
 	gtk_widget_add_controller (GTK_WIDGET (map), g_steal_pointer (&scroll));
+
+	press = GTK_EVENT_CONTROLLER (gtk_gesture_click_new ());
+	g_signal_connect_swapped (press,
+	                          "pressed",
+	                          G_CALLBACK (gtk_source_map_click_pressed),
+	                          map);
+	gtk_event_controller_set_propagation_phase (press, GTK_PHASE_CAPTURE);
+	gtk_widget_add_controller (GTK_WIDGET (map), press);
 }
 
 /**
