@@ -1,8 +1,7 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
- * gtksourceutils.h
+/*
  * This file is part of GtkSourceView
  *
- * Copyright (C) 2013 - Sébastien Wilmet <swilmet@gnome.org>
+ * Copyright 2013 - Sébastien Wilmet <swilmet@gnome.org>
  *
  * GtkSourceView is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,24 +17,21 @@
  * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef GTK_SOURCE_UTILS_H
-#define GTK_SOURCE_UTILS_H
+#pragma once
 
 #if !defined (GTK_SOURCE_H_INSIDE) && !defined (GTK_SOURCE_COMPILATION)
 #error "Only <gtksourceview/gtksource.h> can be included directly."
 #endif
 
 #include <glib-object.h>
-#include <gtksourceview/gtksourceversion.h>
+
+#include "gtksourceversion.h"
 
 G_BEGIN_DECLS
 
 GTK_SOURCE_AVAILABLE_IN_3_10
-gchar		*gtk_source_utils_unescape_search_text		(const gchar	*text);
-
+gchar *gtk_source_utils_unescape_search_text (const gchar *text);
 GTK_SOURCE_AVAILABLE_IN_3_10
-gchar		*gtk_source_utils_escape_search_text		(const gchar	*text);
+gchar *gtk_source_utils_escape_search_text   (const gchar *text);
 
 G_END_DECLS
-
-#endif /* GTK_SOURCE_UTILS_H */
